<?php
namespace Wovnio\Wovnwp\WpClasses;

class HtmlConverter {
	private $html;
	private $accessed_url;
	private $options;
	private $target_lang;

	public function __construct($html, $accessed_url, $options, $target_lang) {
		$this->html = $html;
		$this->accessed_url = $accessed_url;
		$this->options = $options;
		$this->target_lang = $target_lang;
	}

	public function insert_snippet_and_hreflang_tags() {
		$this->html = $this->insert_snippet($this->html);
		$this->html = $this->insert_hreflang_tags($this->html);

		return $this->html;
	}

	/**
	 * Insert wovn's snippet to ensure snippet is always inserted.
	 * If snippet is already inserted, replace it
	 *
	 * @param string $html
	 * @return string
	 */
	private function insert_snippet($html) {
		$snippet_regex = "/<script[^>]*src=[^>]*j\.[^ '\">]*wovn\.io[^>]*><\/script>/i";
		$html = $this->remove_tag_from_html_by_regex($html, $snippet_regex);

		$snippet_code = $this->build_snippet_code();
		$parent_tags = array("(<head\s?.*?>)", "(<body\s?.*?>)", "(<html\s?.*?>)");

		return $this->insert_after_tag($parent_tags, $html, $snippet_code);
	}

	private function insert_after_tag($tag_names, $html, $insert_str) {
		foreach ($tag_names as $tag_name) {
			if (preg_match($tag_name, $html, $matches, PREG_OFFSET_CAPTURE)) {
				return substr_replace($html, $insert_str, $matches[0][1] + strlen($matches[0][0]), 0);
			}
		}

		return $html;
	}

	private function remove_tag_from_html_by_regex($html, $regex) {
		$result = $html;

		if (preg_match_all($regex, $result, $matches, PREG_OFFSET_CAPTURE)) {
			for ($i = count($matches[0]) - 1; $i >= 0; --$i) {
				$match = $matches[0][$i];
				$result = substr_replace($result, '', $match[1], strlen($match[0]));
			}
		}

		return $result;
	}

	private function build_snippet_code() {
		$token = $this->options['wovn_user_token'];
		$current_lang = $this->target_lang;
		$default_lang = $this->options['wovn_default_lang'];
		$url_pattern = $this->options['wovn_url_pattern_name'];
		$lang_param_name = $this->options['wovn_lang_param_name'];
		$wp_debug = defined('WP_DEBUG') && true === WP_DEBUG ? 'true' : 'false';

		# Wordpress doesn't support lang_code_aliases
		$lang_code_aliases_json = '{}';

		return "<script src=\"//j.wovn.io/1\" data-wovnio=\"key=$token&backend=true&currentLang=$current_lang&defaultLang=$default_lang&urlPattern=$url_pattern&langCodeAliases=$lang_code_aliases_json&version=WOVN.wp&debugMode=$wp_debug&langParamName=$lang_param_name\" data-wovnio-type=\"fallback_snippet\" async></script>";
	}

	/**
	 * Insert hreflang tags for all supported_langs
	 *
	 * @param string $html
	 * @return string
	 */
	private function insert_hreflang_tags($html) {
		if (isset($this->options['wovn_supported_langs'])) {
			if (is_array($this->options['wovn_supported_langs'])) {
				$lang_codes = $this->options['wovn_supported_langs'];
			} else {
				$lang_codes = array($this->options['wovn_supported_langs']);
			}
		} else {
			$lang_codes = array();
		}

		$lang_codes_with_pipe = implode('|', $lang_codes);
		$hreflang_regex = "/<link[^>]*hreflang=[^>]*($lang_codes_with_pipe)[^>]*\>/iU";
		$html = $this->remove_tag_from_html_by_regex($html, $hreflang_regex);
		$url_pattern = $this->options['wovn_url_pattern_name'];
		$default_lang = $this->options['wovn_default_lang'];
		$lang_param_name = $this->options['wovn_lang_param_name'];

		$hreflangTags = array();
		foreach ($lang_codes as $lang_code) {
			$href = htmlentities(Url::add_lang_code($this->accessed_url, $url_pattern, $lang_code, $default_lang, $lang_param_name));
			array_push($hreflangTags, '<link rel="alternate" hreflang="' . Lang::iso639_1_normalization($lang_code) . '" href="' . $href . '">');
		}

		$parent_tags = array("(<head\s?.*?>)", "(<body\s?.*?>)", "(<html\s?.*?>)");

		return $this->insert_after_tag($parent_tags, $html, implode('', $hreflangTags));
	}
}
