<?php
namespace Wovnio\Wovnwp\WpClasses;
use Wovnio\Wovnwp\Constants;
require_once( 'lang.php' );

class Url {
	/**
	 * Adds a language code to a uri.
	 *
	 * @param string  $absolute_url     The uri to modify, assuming there is no lang code.
	 * @param string $url_pattern
	 * @param string  $lang_code    The language code to add to the uri.
	 *
	 * @return string The new uri containing the language code.
	 */
	public static function add_lang_code($absolute_url, $url_pattern, $lang_code, $default_lang, $lang_param_name = null ) {
		if (!$lang_code || strlen($lang_code) == 0) {
			return $absolute_url;
		}

		if ($default_lang === $lang_code) {
			return $absolute_url;
		}

		$parsed_url = parse_url($absolute_url);
		// On seriously malformed URLs, parse_url() may return FALSE. (php doc)
		if (!$parsed_url || !array_key_exists('host', $parsed_url)) {
			return $absolute_url;
		}

		switch ($url_pattern) {
			case 'subdomain':
				$new_uri = preg_replace('/(\/\/)([^\.]*)/', '${1}' . self::format_for_reg_exp(strtolower($lang_code)) . '.' . '${2}', $absolute_url, 1);
				break;
			case 'query':
				if (empty($lang_param_name)) $lang_param_name = Constants::$default_lang_param_name;
				$new_uri = self::add_query_lang_code($absolute_url, $lang_code, $lang_param_name);
				break;
			default:
				//path
				$new_uri = preg_replace('/([^\.]*\.[^?\/]*)(\?|\/|$)/', '${1}/' . self::format_for_reg_exp($lang_code) . '${2}', $absolute_url, 1);
		}

		return $new_uri;
	}

	public static function path_lang_code($absolute_url, $url_pattern, $lang_param_name = null)
	{
		switch ($url_pattern) {
			case 'subdomain':
				preg_match('/\/\/(' . join('|', array_keys(Constants::$langs)) .')\./', $absolute_url, $matches);
				break;
			case 'query':
				if (empty($lang_param_name)) $lang_param_name = Constants::$default_lang_param_name;
				preg_match('/' . $lang_param_name . '=(' . join('|', array_keys(Constants::$langs)) .')/', $absolute_url, $matches);
				break;
			default:
				//path
				preg_match('/\/(' . join('|', array_keys(Constants::$langs)) .')\//', $absolute_url, $matches);
		}
		return $matches && $matches[1] ? $matches[1] : Lang::browser_lang();
	}

	private static function format_for_reg_exp($text) {
		return str_replace('$', '\$', str_replace("\\", "\\\\", $text));
	}

	/**
	 * Adds a language code to a uri using query pattern.
	 *
	 * @param String  $uri     The uri to modify.
	 * @param String  $lang_code    The language code to add to the uri.
	 *
	 * @return String The new uri containing the language code.
	 */
	private static function add_query_lang_code($uri, $lang_code, $lang_param_name) {
		$sep = '?';
		if (preg_match('/\?/', $uri)) {
			$sep = '&';
		}
		return preg_replace('/(#|$)/', $sep . $lang_param_name . '=' . $lang_code . '${1}', $uri, 1);
	}
}
